
/*
 * KENDALL
 * ctest.c -- test program for kendall generated parsers (includes some utility functions)
 * $Id: ctest.c 54 2005-09-11 14:42:49Z avatar $
 *
 */

#include "kendall.h"  /* struct kendall_node_t */
#include "chicken.h"

/* Constructor/destructor for kendall_node_t struct */
NODE_PTR
make_node(char *op, int label, NODE_PTR child)
{
	NODE_PTR node = malloc(sizeof(struct kendall_node_t));

	if (node != NULL) {
		node->op = malloc(strlen(op) + 1);

		if (node->op == NULL) {
			free(node);
			return(NULL);
		}

		strcpy(node->op, op);
		node->label = label;
		node->child = child;
	}

	return(node);
}

void
free_node(NODE_PTR node)
{
	free(node->op);
	free(node);
}

/* Functions defined in matcher.scm (compiled to matcher.c) */
extern void     match(NODE_PTR);
extern void     read_tables(void);


/* TEST PROGRAM*/

int
main(int argc, char *argv[])
{
	/* Must be called before calling match() for the first time */
	read_tables();

	fprintf(stdout, "FIRST TEST RUN\n");
	NODE_PTR leaf_1 = make_node("s2",    -1, NULL);
	NODE_PTR node_1 = make_node("iload", -1, leaf_1);
	NODE_PTR root_1 = make_node("iload", -1, node_1);
	match(root_1); /* label(root); reduce(root) */

	fprintf(stdout, "SECOND TEST RUN\n");
	NODE_PTR leaf_2 = make_node("s2",    -1, NULL);
	NODE_PTR node_2 = make_node("iload", -1, leaf_2);
	NODE_PTR root_2 = make_node("iadd",  -1, node_2);
	match(root_2);

	fprintf(stdout, "THIRD TEST RUN\n");
	NODE_PTR leaf_3 = make_node("s0",     -1, NULL);
	NODE_PTR node_3 = make_node("istore", -1, leaf_3);
	NODE_PTR root_3 = make_node("istore", -1, node_3);
	match(root_3);

	fprintf(stdout, "FOURTH TEST RUN\n");
	NODE_PTR leaf_4 = make_node("s0",     -1, NULL);
	NODE_PTR node_4 = make_node("istore", -1, leaf_4);
	NODE_PTR root_4 = make_node("iload",  -1, node_4);
	match(root_4);

	fprintf(stdout, "FIFTH TEST RUN\n");
	NODE_PTR leaf_5 = make_node("s0",    -1, NULL);
	NODE_PTR node_5 = make_node("iload", -1, leaf_5);
	NODE_PTR root_5 = make_node("iload", -1, node_5);
	match(root_5);

	fprintf(stdout, "SIXTH TEST RUN\n");
	NODE_PTR leaf_6 = make_node("s1",   -1, NULL);
	NODE_PTR node_6 = make_node("iadd", -1, leaf_6);
	NODE_PTR root_6 = make_node("iadd", -1, node_6);
	match(root_6);

	fprintf(stdout, "SEVENTH TEST RUN\n");
	NODE_PTR leaf_7  = make_node("s2",     -1, NULL);
	NODE_PTR node1_7 = make_node("iload",  -1, leaf_7);
	NODE_PTR node2_7 = make_node("istore", -1, node1_7);
	NODE_PTR node3_7 = make_node("iload",  -1, node2_7);
	NODE_PTR node4_7 = make_node("iadd",   -1, node3_7);
	NODE_PTR node5_7 = make_node("istore", -1, node4_7);
	NODE_PTR node6_7 = make_node("istore", -1, node5_7);
	NODE_PTR root_7  = make_node("istore", -1, node6_7);
	match(root_7);

	fprintf(stdout, "EIGHTH TEST RUN\n");
	NODE_PTR leaf_8  = make_node("s0",     -1, NULL);
	NODE_PTR node1_8 = make_node("iload",  -1, leaf_8);
	NODE_PTR node2_8 = make_node("iload",  -1, node1_8);
	NODE_PTR node3_8 = make_node("iadd",   -1, node2_8);
	NODE_PTR root_8  = make_node("istore", -1, node3_8);
	match(root_8);

	fprintf(stdout, "NINTH TEST RUN\n");
	NODE_PTR leaf_9  = make_node("s1",     -1, NULL);
	NODE_PTR node1_9 = make_node("iload",  -1, leaf_9);
	NODE_PTR node2_9 = make_node("iload",  -1, node1_9);
	NODE_PTR node3_9 = make_node("iadd",   -1, node2_9);
	NODE_PTR root_9  = make_node("istore", -1, node3_9);
	match(root_9);

	fprintf(stdout, "TENTH TEST RUN\n");
	NODE_PTR leaf_0  = make_node("s2",     -1, NULL);
	NODE_PTR node1_0 = make_node("iload",  -1, leaf_0);
	NODE_PTR node2_0 = make_node("iload",  -1, node1_0);
	NODE_PTR node3_0 = make_node("iadd",   -1, node2_0);
	NODE_PTR root_0  = make_node("istore", -1, node3_0);
	match(root_0);


	fprintf(stdout, "ELEVENTH TEST RUN\n");
	NODE_PTR leaf_11  = make_node("s2",     -1, NULL);
	NODE_PTR root_11  = make_node("istore", -1, leaf_11);
	match(root_11);

	fprintf(stdout, "TWELFTH TEST RUN\n");
	NODE_PTR leaf_12  = make_node("s2",     -1, NULL);
	NODE_PTR root_12  = make_node("iadd", -1, leaf_12);
	match(root_12);

	fprintf(stdout, "THIRTEENTH TEST RUN\n");
	NODE_PTR leaf_13  = make_node("s1",     -1, NULL);
	NODE_PTR root_13  = make_node("istore", -1, leaf_13);
	match(root_13);

	fprintf(stdout, "FOURTEENTH TEST RUN\n");
	NODE_PTR leaf_14  = make_node("s1",     -1, NULL);
	NODE_PTR root_14  = make_node("iload", -1, leaf_14);
	match(root_14);


	fprintf(stdout, "FIFTEENTH TEST RUN\n");
	NODE_PTR leaf_15  = make_node("s2",     -1, NULL);
	NODE_PTR node_15  = make_node("istore", -1, leaf_15);
	NODE_PTR root_15  = make_node("istore", -1, node_15);
	match(root_15);

	return(0);
}

/* EOF */
