
;;;
;;; KENDALL
;;; main.scm -- main program loop
;;; $Id: main.scm 54 2005-09-11 14:42:49Z avatar $
;;;

(require-extension args)
(require-extension format)

(declare (uses regex))
(declare (uses extras))
(declare (uses utils))
(declare (uses srfi-1))

(include "util.scm")
(include "struct.scm")
(include "burs.scm")
(include "parsegen.scm")

;; Internal representation of input grammar
(define rules '())                      ; All rules
(define chainrules '())
(define non-crs '())
(define nonterminals '())
(define operators '())
;; Tables for BURS automata
;; and/or use in table generation algorithms
(define worklist '())                   ; States to be processed
(define leaf-states '())                ; (leaf . state)
(define states '()) ;(make-hash-table string-ci=?))
(define op-map '())
(define op-reps '())
(define transition-table '())
(define rules-table '())
(define vrules '#())

;;  ________
;; < MAIN() >
;;  --------
;;   \            .    .     .
;;    \      .  . .     `  ,
;;     \    .; .  : .' :  :  : .
;;      \   i..`: i` i.i.,i  i .
;;       \   `,--.|i |i|ii|ii|i:
;;            UooU\.'@@@@@@`.||'
;;            \__/(@@@@@@@@@@)'
;;                 (@@@@@@@@)
;;                 `YY~~~~YY'
;;                  ||    ||

(letrec ([debug #f]
	 [unixy #f]
	 [hardwire #f]
	 [input-file-name ""]
	 [opts
	  (list (args:make-option (f file)     (required: "NAME")  "Parse file NAME")
		(args:make-option (d debug)    #:none "Enable debugging output")
		(args:make-option (w hardwire) #:none "Output hardwired parsers")
		(args:make-option (p print)    #:none "Print output file in unixy format")
		(args:make-option (v version)  #:none "Display version information"
				  (begin
				    (print "KENDALL, v. 0.0.1")
				    (exit 0)))
		(args:make-option (h help)     #:none "Display this text"
				  (usage)))]
	 [usage (lambda ()
		  (with-output-to-port (current-error-port)
		    (lambda ()
		      (print "usage: " (car (argv)) " [options] file")
		      (newline)
		      (print (args:usage opts))))
		  (exit 1))])

  ;; Command line options...
  (receive (options operands)
	   (args:parse (command-line-arguments) opts)
	   (set! input-file-name (alist-ref 'file options))
	   (if (assq 'debug options)
	       (set! debug #t))
	    (if (assq 'hardwire options)
	       (set! hardwire #t))
	   (if (assq 'print options)
	       (set! unixy #t)))

  ;; Parse input file
  (if (not (file-exists? input-file-name))
      (kendall-error (conc "File not found -- " input-file-name))
      (let ([p (open-input-file input-file-name)])
	(for-each-line parse-input-line p)
	(close-input-port p)))

  (set! rules (parse-input-line '()))

  ;; Build lists of operators, non-terminals and so on
  (set! nonterminals (get-nonterminals rules '()))
  ;(set! rules (append rules (close-rules (apply max (map number rules)))))
  (set! rules (append rules (close-rules (my-max-rno rules))))

  (set! nonterminals (cons "kendall" nonterminals))
  (set! operators (get-operators rules '()))
  (set! chainrules (get-chainrules rules))
  (set! non-crs (get-non-chainrules rules))

  (set! op-reps (make-table operators '()))
  (set! op-map (make-table operators '()))
  (set! transition-table (make-table operators '()))


  ;;NEW
  (init-vrules (my-max-rno rules))
  (init-rhs-table (my-max-rno rules))
  (init-rboa)
  ;(init-nt)

  ;; Generate BURS tables
  (compute-leaf-states nonterminals)
  (table-gen)
  (set! rules-table (make-rules-table))

  ;; Output BURS tables
  (cond (unixy (print-kendall-tables-unixy))
	(hardwire (generate-parser))
	(else
	 (print-kendall-tables)))

  ;; Debugging output
  (if debug
      (begin
	(printf "~%BEGIN DEBUGGING OUTPUT~%")

	(printf "Parsing input file...~%")
	(printf "All rules:~%")
	(print rules)
	(printf "Chainrules:~%")
	(print chainrules)
	(printf "Non-chainrules:~%")
	(print non-crs)
	(printf "Operators:~%")
	(print operators)
	(printf "Nonterminals:~%")
	(print nonterminals)

	(printf "~%Generating BURS tables...~%")
	(printf "States:~%")
	(print states)
	(printf "Leaf states:~%")
	(print leaf-states)
	(printf "Transitions Table:~%")
	(print transition-table)
	(printf "Rules table:~%")
	(print rules-table)
	(printf "op.map:~%")
	(print op-map)
	(printf "op.reps:~%")
	(print op-reps)
        (print rhs-table)
        (print vrules)
	(printf "~%END DEBUGGING OUTPUT~%")))
  )

;;; EOF
