
;;;
;;; KENDALL
;;; matcher.scm -- BURS pattern matcher
;;; $Id: matcher.scm 52 2005-09-11 12:45:39Z avatar $
;;;

#>
#include "kendall.h"
<#

(include "chicken-entry-points")

(declare (uses regex))
(declare (uses extras))
(declare (uses utils))
(include "util.scm")
(include "struct.scm")

;; Read input file.
(define rules '())
(define states '())
(define leaf-states '())
(define op-map '())
(define transition-table '())
(define rules-table '())
(define vrules '#())



;; C-Interface
(define-foreign-record (node "struct kendall_node_t")
  (c-string op)
  (int label)
  (node child))

(define-embedded (match
                  ((pointer
                    (struct
                     "kendall_node_t")) n)) void
                     (label n)
                     (reduce n))

(define-embedded (read_tables) void ;(define (read-tables)
  (call-with-values
      (lambda () (read-kendall-tables))
      (lambda (a b c d e f)
        (set! rules a)
	(set! states b)
	(set! leaf-states c)
	(set! op-map d)
	(set! transition-table e)
	(set! rules-table f)
        (init-vrules (my-max-rno rules))
        )))




;; NODE: string node int
; (define-record node
;   op
;   child
;   label)

; (define-record-printer (node x out)
;   (cond ((eq? (node-child x) 'none)
; 	 (fprintf out "~S (~S)~%"
; 		  (node-op x)
; 		  (node-label x)))
; 	(else
; 	 (fprintf out "~S (~S)~%  |~%~S"
; 		  (node-op x)
; 		  (node-label x)
; 		  (node-child x)))))

;;  _________
;; < LABELER >
;;  ---------
;;         \    ,-^-.
;;          \   !oYo!
;;           \ /./=\.\______
;;                ##        )\/\
;;                 ||-----w||
;;                 ||      ||
;;
;; Reads global lists 'leaf-states', 'transition-table', 'op-map'.
;; Mutates its argument (sets label field).
;; Arg: Root of parse tree
(define (label node)
  (let ([c (node-child node)]
	[o (node-op node)])
    (if (eq? c #f) ; 'none
	(node-label-set!
	 node
	 (cdr (assoc o leaf-states)))
	(begin
	  (label c)
	  (node-label-set!
	   node
	   (transition-table-lookup
	    o
	    (state-to-repstate o (node-label c))))))))

;;  _________
;; < REDUCER >
;;  ---------
;;         \   ^__^
;;          \  (==)\_______
;;             (__)\       )\/\
;;                 ||----w |
;;                 ||     ||
;;
;; Reads global lists 'rules', 'states', 'rules-table'.
;; Prints to stdout.
;; Args: Root of parse tree, optionally goal nonterminal
(define (reduce root . g)

  (define (fire rule)
    (printf (action rule))
    (newline))

(define (reduce-tree node nonterm)
  (letrec ([state (state-reverse-lookup (node-label node))]
           [op    (node-op node)]
           [chld  (node-child node)]
           [rlist (reduce-node state op nonterm)]
           [get-nnt (lambda (l)
                      (cond ((null? (cdr l))
                             (car l))
                            (else
                             (get-nnt (cdr l)))))]
	   [nnt (if (not (null? rlist)) (rhs-arg (get-nnt rlist)))]) ; next nonterm  (rhs-arg (rule-lookup (get-nnt rlist))))])

    (if (eq? chld #f) ; 'none
	rlist                           ; List of rules to be fired
	(append rlist
		(reduce-tree chld nnt)))))

  (define (reduce-node state op nonterm)
    (letrec ([loop (lambda (nt)
                     (let* ([rule (rule-lookup (state-rule state nt))]
                            [rule-rhs (rhs-op (number rule))])
                       (cond ((equal? op nonterm)
                              '())
                             ((equal? rule-rhs op)
                              (cons (number rule) '()))
                             (else
                              (cons (number rule)
                                    (loop rule-rhs))))))]) ;; arg should always be a nonterm...
      (loop nonterm)))

  (letrec ([goal (:optional g
			    (rhs (rule-lookup (state-rule (state-reverse-lookup (node-label root)) "kendall"))))]
				  ;(rules-table-lookup "kendall"
				;		      (node-label root)))))]
	   [result (reverse (reduce-tree root goal))])

    (for-each fire (map rule-lookup result))
    (newline)))

;;; EOF
