
;;;
;;; KENDALL
;;; burs.scm -- table generation algorithms
;;; $Id: burs.scm 54 2005-09-11 14:42:49Z avatar $
;;;

;; FIXME: Check divergent...
;; Reads global list 'nonterminals'
;; Mutates its argument
;; Arg: State object
(define (normalize-costs state)
  (letrec ([delta (state-min-cost state)]
	   [loop (lambda (x)
		   (state-cost-set!
		    state x (inf-minus (state-cost state x) delta)))])
    (for-each loop nonterminals)))

;; Reads global list 'chainrules'
;; Mutates its argument
;; Args: State object, flag
(define (closure state changes)
  (if changes
      (letrec ([loop (lambda (cr)
		       (let ([cost (inf-plus (cost cr)
					     (state-cost state (rhs cr)))])
			 (if (inf-lt cost (state-cost state (lhs cr)))
			     (begin
			       (state-entry-set!
				state (lhs cr) (make-rcp (number cr) cost))
			       (set! changes #t)))))])
	(set! changes #f)
	(for-each loop chainrules)
	(closure state changes))))

;; state->rstate
;; Reads global lists 'nonterminals', 'states', 'rules'
;; Args: Operator, index into 'states'
(define (project op st)
  (letrec ([state (state-reverse-lookup st)]
	   [pstate (make-state)]        ; Repstate
	   [loop (lambda (nt)
		   (if   (hash-table-ref rules-by-op/arg (conc op nt)) ;(find-rule op nt)
		       (state-cost-set! pstate nt (state-cost state nt))))])
    (for-each loop nonterminals)
    (normalize-costs pstate)
    pstate))

;;  ___________________
;; < State Computation >
;;  ------------------
;;           \      (__)
;;            \     /oo|
;;             \   (_"_)*+++++++++*
;;                    //I#\\\\\\\\I\
;;                    I[I|I|||||I I `
;;                    I`I'///'' I I
;;                    I I       I I
;;                    ~ ~       ~ ~
;;
;; compute-leaf-states generates the initial states (which are then added to
;; the global list 'worklist') and must be called before table-gen.
;;
;; table-gen drives the state computation process by iterating over worklist
;; and calling compute-transitions for every operator/state combination until
;; compute-transitions adds no more new states to worklist.
;;
;; normalize-costs, closure, and project are helper functions used by both
;; compute-leaf-states and compute-transitions.
;;

;; Reads global list 'chainrules'
;; Assigns global lists 'states', 'worklist', 'leaf-states'
;; Arg: List of possible leaf operators (in our case => nonterminals)
(define (compute-leaf-states leaves)
  (letrec ([leaf-rule? (lambda (leaf)
			 (lambda (r)
			   (equal? (rhs r) leaf)))]
	   [outer (lambda (leaf)
		    (let* ([state (make-state)]
			   [leaf-rules (filter
					(leaf-rule? leaf)
					chainrules)]
			   [inner (lambda (r)
				    (if (inf-lt (cost r) (state-cost state (lhs r)))
					(state-entry-set! state (lhs r) (make-rcp (number r) (cost r)))))])
		     (for-each inner leaf-rules)
		     (normalize-costs state)
		     (closure state #t)
		     (let ([n (state-install! state)])
		       (set! worklist (append worklist (list n)))
		       (set! leaf-states (cons
					  (cons leaf n)
					  leaf-states)))))])
    (for-each outer leaves)))

;; Reads global list 'operators'.
;; Assigns global list 'worklist'.
(define (table-gen)
  (if (null? worklist)
      'done
      (letrec ([state (car worklist)]   ; Index into states
	       [loop (lambda (op)
		       (compute-transitions op state))])
	(set! worklist (cdr worklist))  ; Pop...
	(for-each loop operators)
	(table-gen))))

;; FIXME: op-reps need not be global
;; Reads global lists 'op-reps', 'states', 'non-crs'.
;; Assigns global lists 'op-reps', 'states', 'worklist', 'transition-table', 'op-map'.
;; Args: Operator, index into 'states'
(define (compute-transitions op state)
  (letrec ([pstate (project op state)]
           [pstate-index (op-reps-lookup op pstate)]
           [result (make-state)]
           [result-index 'BAKABAKA]
           [rs (filter (lambda (r)
                         (equal? (rhs-op (number r)) op))
                       non-crs)]
           [fun (lambda (r)
                  (letrec ([nt (rhs-arg (number r))]
                           [c (inf-plus (cost r) (state-cost pstate nt))])
                    (if (inf-lt c (state-cost result (lhs r)))
                        (state-entry-set! result (lhs r) (make-rcp (number r) c)))))])

    (if (eq? pstate-index #f)
        (begin
          (set! pstate-index (op-reps-install! op pstate))
          (for-each fun rs)
          (normalize-costs result)

          (set! result-index (state-lookup result))
          (if (eq? result-index #f)
              (begin
                (closure result #t)
                (set! result-index (state-install! result))
                (set! worklist (append worklist (list result-index)))))

          (transition-table-install! op pstate-index result-index)))

    (op-map-install! op state pstate-index)))

;;; EOF
