
;;;
;;; KENDALL
;;; parsegen.scm -- code for generating hardwired parsers
;;; $Id: parsegen.scm 52 2005-09-11 12:45:39Z avatar $
;;;

;; Uses BIG-LIST-O-CONDS (TM) technology

;;; Labeler...
(define (generate-labeler p)

  (define (generate-first-cond)
    (let ([quux (lambda (nt)
		  `((equal? o ,nt)
		    (node-label-set!
		     node
		     ,(cdr (assoc nt leaf-states)))))])
      (map quux nonterminals)))

  (define (generate-second-cond)
    (let ([foo (lambda (op)
		 (let ([baz (lambda (label)
			      `((equal? (node-label c)
					,label)
				(node-label-set!
				 node
				 ,(transition-table-lookup
				   op
				   (state-to-repstate op label)))))])
		   `((equal? o ,op)
		     ,(cons 'cond (map baz (map cdr states))))))])
      (map foo operators)))

  (let* ([1st-cond (generate-first-cond)]
	 [2nd-cond (generate-second-cond)]
	 [labeler `(define (label node)
		     (let ([l (node-label node)]
			   [c (node-child node)]
			   [o (node-op node)])
		       (if (eq? c 'none)
			   (cond
			    ,@1st-cond)
			   (begin
			     (label c)
			     (cond
			      ,@2nd-cond)))))])

    (pretty-print labeler p)))

;;; Reducer...
(define (generate-reducer p)

  (define (generate-first-cond)
    (let ([bar (lambda (r)
		 `((equal? ,(number r) last)
		   (set! nnt ,(rhs-arg (number r)))))])
      (map bar rules)))

  (define (generate-second-cond)
    (let ([quux (lambda (stateno)
		  (let* ([state (state-reverse-lookup stateno)]
			 [baz (lambda (nt)
				(let* ([rule (rule-lookup (state-rule state nt))]
				       [rh 'BAKA]
				       [num 'BAKA])
				  (if rule
				      (begin
					(set! rh (rhs-op (number rule)))
					(set! num (number rule))))

				  `((equal? nonterm ,nt)
				    (cond
				     ((equal? (node-op node)
					      ,rh)
				      (cons ,num '()))
				     (else
				      (cons ,num
					    (reduce-node node ,rh)))))))])
		    `((equal? (node-label node) ,stateno)
		      ,(cons 'cond (map baz nonterminals)))))])

      (map quux (map cdr states))))

  (define (generate-third-cond)
    (let ([foo (lambda (label)
		 (let ([rule (rule-lookup (rules-table-lookup "kendall" label))]
                       [rh 'BAKA])
		   (if rule
		       (set! rh (rhs rule)))

		 `((equal? (node-label root) ,label)
		   ,rh)))])

      (map foo (map cdr states))))

  (define (generate-fourth-cond)
    (let ([quux (lambda (rule)
		  `((equal? (car list) ,(number rule))
		    (display ,(action rule))
		    (newline)))])

      (map quux rules)))

  (let* ([1st-cond (generate-first-cond)]
	 [2nd-cond (generate-second-cond)]
	 [3rd-cond (generate-third-cond)]
	 [4th-cond (generate-fourth-cond)]
	 [reducer `(define (reduce root . g)

		     (define (reduce-tree node nonterm)
		       (letrec ([rlist (reduce-node node nonterm)]
				[get-nnt (lambda (l)
					   (if (null? (cdr l))
					       (car l)
					       (get-nnt (cdr l))))]
				[nnt 'GAGA]
				[last 'GAGA])
			 (if (not (null? rlist))
			     (begin
			       (set! last (get-nnt rlist))
			       (cond
				,@1st-cond)))
			 (if (eq? (node-child node) 'none)
			     rlist
			     (append rlist
				     (reduce-tree (node-child node) nnt)))))

		     (define (reduce-node node nonterm)
		       (cond ((equal? (node-op node) nonterm) '())
			     (else
			      (cond ,@2nd-cond))))

		     (let* ([goal (:optional g
					     (cond ,@3rd-cond))]
			    [result (reverse (reduce-tree root goal))])

		       (let loop ([list result])
			 (if (null? list)
			     'done
			     (begin
			       (cond
				,@4th-cond)
			       (loop (cdr list)))))))])

	 (pretty-print reducer p)))

;;; Parser...
(define (generate-parser)
  (let ([p (open-output-file "kendall.scm")]
	[node-def '(define-record node op child label)]
	[node-printer '(define-record-printer (node x out)
			 (cond ((eq? (node-child x) 'none)
				(fprintf out "~S (~S)~%"
					 (node-op x)
					 (node-label x)))
			       (else
				(fprintf out "~S (~S)~%  |~%~S"
					 (node-op x)
					 (node-label x)
					 (node-child x)))))])
    (newline p)
    (display ";;; generated by kendall, do not modify" p)
    (newline p)
    (pretty-print node-def p)
    (newline p)
    (pretty-print node-printer p)
    (newline p)
    (generate-labeler p)
    (newline p)
    (generate-reducer p)
    (newline p)
    (display ";;; EOF" p)
    (newline p)
    (close-output-port p)))

;;; EOF
