
;;;
;;; KENDALL
;;; struct.scm -- major data structures and relevant helper functions
;;; $Id: struct.scm 47 2005-09-08 13:48:40Z avatar $
;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; RULES ;;;;

;; RULE: Proper list with elements (number lhs rhs cost action)
(define (number rule)
  (car rule))

(define (lhs rule)
  (cadr rule))

(define (rhs rule)
  (caddr rule))

; (define (rhs-op rule)
;   (let ([op (operator? (rhs rule))])
;     (if (not op)
; 	(rhs rule)
; 	(cadr op))))

; (define (rhs-arg rule)
;   (let ([op (operator? (rhs rule))])
;     (if (not op)
; 	(rhs rule)
; 	(caddr op))))

(define (rhs-op rno)
  (car (vector-ref rhs-table rno)))

(define (rhs-arg rno)
  (cdr (vector-ref rhs-table rno)))

(define rhs-table '#())

;; vecor indexed by rule nos contains (rhs-op . rhs-arg) pairs
(define (init-rhs-table m)
  (let ((pp-rhs (lambda (r)
                  (let* ((right (rhs r))
                         (temp  (operator? right))
                         (op (if (not temp)
                                 right
                                 (cadr temp)))
                         (arg (if (not temp)
                                  right
                                  (caddr temp))))

                    (vector-set! rhs-table (number r) (cons op arg))))))

    (set! rhs-table (make-vector (+ m 1)))
    (for-each pp-rhs rules)))



(define (cost rule)
  (cadddr rule))

(define (action rule)
  (car (cddddr rule)))

(define (my-max-rno rs)
  (if (null? (cdr rs))
      (number (car rs))
      (my-max-rno (cdr rs))))

;; Search global list 'rules' for a rule with number 'no' and return that rule if found, else #f.
;; Reads global list rules
(define (init-vrules m)
  (begin
    (set! vrules (make-vector (+ m 1)))
    (for-each (lambda (r) (vector-set! vrules (number r) r))
              rules)))


(define (rule-lookup no)
  (if (not (number? no))
      #f
      (vector-ref vrules no)))
;   (letrec ([loop (lambda (l)
; 		   (if (null? l)
; 		       #f
; 		       (if (= no (number (car l)))
; 			   (car l)
; 			   (loop (cdr l)))))])
;     (loop rules)))

(define (get-operators rules operators)
  (if (null? rules)
      operators
      (let* ([op (operator? (rhs (car rules)))])
	(if (and op
		 (not (member (cadr op) operators)))
	    (get-operators (cdr rules) (cons (cadr op) operators))
	    (get-operators (cdr rules) operators)))))

(define (get-nonterminals rules nonterms)
  (if (null? rules)
      nonterms
      (let ([lhs (cadr (car rules))])
	(if (not (member lhs nonterms))
	    (get-nonterminals (cdr rules) (cons lhs nonterms))
	    (get-nonterminals (cdr rules) nonterms)))))

(define (get-chainrules rules)
  (filter chain-rule? rules))

(define (get-non-chainrules rules)
  (filter (lambda (x)
	   (not (chain-rule? x)))
	 rules))

;; if arg matches foo(bar) returns list of matched groups else #f
(define (operator? op)
  (string-match "([A-Za-z][A-Za-z0-9]*)\\(([A-Za-z][A-Za-z0-9]*)\\)" op))

(define (chain-rule? rule)
  (not (operator? (rhs rule))))



; (define nt-to-num (make-hash-table string-ci=?))

; (define (init-nt)
;   (let ((num 0))

;     (for-each (lambda (nt) (hash-table-set! nt-to-num (conc "_" nt)
;                                        (let ((n num)) (set! num (+ num 1)) n)))
;               nonterminals)))


(define rules-by-op/arg (make-hash-table string-ci=?))

(define (init-rboa)
  (let ((pairs (let loop ((ops operators) (result '()))
                 (if (null? ops)
                     result
                     (loop (cdr ops) (append (map (lambda (nt) (cons (car ops) nt)) nonterminals)
                                             result))))))


    (for-each (lambda (p) (hash-table-set! rules-by-op/arg (conc (car p) (cdr p))
                                      (find-rule-old (car p) (cdr p))))
              pairs)))

;; Find rule with rhs op(nt)
;; Reads global list 'rules'.
(define (find-rule-old op nt)
  (let* ([l1 (filter
	      (lambda (r)
		(equal? (rhs-op (number r)) op))
	      non-crs)]
	 [l2 (filter
	      (lambda (r)
		(equal? (rhs-arg (number r)) nt))
	      l1)])
    (if (null? l2)
	#f
        #t)))
	;l2

;; Generates rules for the 'kendall' nonterminal
;; and 'stay' rules for every nonterminal.
;; Reads global list 'nonterminals'.
(define (close-rules m)
  (letrec ([loop-1 (lambda (l c)
		     (if (null? l)
			 (begin
			   (set! m c)
			   '())
			 (cons
			  (list (+ c 1)
				"kendall"
				(car l)
				0
				"KENDALL")
			  (loop-1 (cdr l) (+ c 1)))))]
	   [loop-2 (lambda (l c)
		     (if (null? l)
			 '()
			 (cons
			  (list (+ c 1)
				(car l)
				(car l)
				0
				"STAY")
			  (loop-2 (cdr l) (+ c 1)))))]
	   [list-1 (loop-1 nonterminals m)]
	   [list-2 (loop-2 nonterminals m)])
    (append list-1 list-2)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; STATES ;;;;;

;; RULE-COST PAIR: (int . int)
(define-record rcp
  rule
  cost)

(define-record-printer (rcp x out)
  (fprintf out "#,(rcp ~S ~S)"
	   (rcp-rule x)
 	   (rcp-cost x)))

(define-reader-ctor 'rcp make-rcp)



; (define (make-state)
;   (let* ((len (length nonterminals))
;          (this (make-vector len)))

;     (for-each (lambda (nt) (vector-set! this (hash-table-ref nt-to-num (conc "_" nt)) (make-rcp 'na 'inf)))
;               nonterminals)

;     this))

; (define (state-eq? s1 s2)
;   (let ((len (length nonterminals)))

;     (let loop ((i 0))
;       (if (>= i len)
;           #t
;           (if (equal? (vector-ref s1 i)
;                       (vector-ref s2 i))
;               (loop (+ i 1))
;               #f)))))

; (define (state-cost state nonterm)
;   (let ((idx (hash-table-ref nt-to-num (conc "_" nonterm))))
;     (rcp-cost (vector-ref state idx))))

; (define (state-rule state nonterm)
;   (let ((idx (hash-table-ref nt-to-num (conc "_" nonterm))))
;     (rcp-rule (vector-ref state idx))))

; (define (state-cost-set! state nonterm newval)
;   (let ((idx (hash-table-ref nt-to-num (conc "_" nonterm))))
;     (rcp-cost-set! (vector-ref state idx) newval)))

; (define (state-rule-set! state nonterm newval)
;   (let ((idx (hash-table-ref nt-to-num (conc "_" nonterm))))
;     (rcp-rule-set! (vector-ref state idx) newval)))

; (define (state-entry-set! state nonterm newval)
;   (let ((idx (hash-table-ref nt-to-num (conc "_" nonterm))))
;     (vector-set! state idx newval)))

; (define (state-min-cost state)
;   (let ((len  (length nonterminals))
;         (costs '()))

;     (let loop ((i 0))
;       (if (>= i len)
;           'done
;           (begin
;             (set! costs (cons (rcp-cost (vector-ref state i)) costs))
;             (loop (+ i 1)))))

;     (inf-min costs)))

;; STATE: alist of (nonterminal . rcp) pairs
;; Reads global list 'nonterminals'.
(define (make-state)
  (letrec ([loop (lambda (l)
		   (if (null? l)
		       '()
		       (cons (cons (car l) (make-rcp 'na 'inf))
			     (loop (cdr l)))))])
    (loop nonterminals)))

(define (state-eq? s1 s2)
  (cond ((null? s1)
	 (if (null? s2) #t #f))
	((equal? (car s1) (car s2))
	 (state-eq? (cdr s1) (cdr s2)))
	(else #f)))

(define (state-cost state nonterminal)
  (rcp-cost (cdr (assoc nonterminal state))))

(define (state-rule state nonterminal)
  (rcp-rule (cdr (assoc nonterminal state))))

(define (state-cost-set! state nonterminal newval)
  (rcp-cost-set! (cdr (assoc nonterminal state)) newval))

(define (state-rule-set! state nonterminal newval)
  (rcp-rule-set! (cdr (assoc nonterminal state)) newval))

(define (state-entry-set! state nonterminal newval)
  (set-cdr! (assoc nonterminal state) newval))

(define (state-min-cost state)
  (letrec ([get-costs (lambda (l)
			(if (null? l)
			    '()
			    (cons (rcp-cost (cdar l))
				  (get-costs (cdr l)))))])
    (inf-min (get-costs state))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; INDEX MAPS ;;;;;

;; INDEX-MAP: Map KEY to INDEX (non-negative integer)

;; Add (key . lastindex+1) to map,
;; return new index and new map (for use with set!)
(define (index-map-install! map key)
  (let ([result 0])
    (if (null? map)
	(set! map
	      (cons (cons key 0) map))
	(let ([n (cdar map)])           ; last index
	  (set! map
		(cons
		 (cons key (+ n 1)) map))
	  (set! result (+ n 1))))
    (values map result)))               ; multiple return values

;; Return integer index
(define (index-map-lookup map key testfun)
  (alist-ref key map testfun))

;; Return key object
(define (index-map-reverse-lookup map val)
  (letrec ([loop (lambda (alist)
		   (if (null? alist)
		       #f
		       (if (= (cdar alist) val)
			   (caar alist)
			   (loop (cdr alist)))))])
    (loop map)))

;; Returns a list of all the indeces in the index map 'map'
(define (index-map->index-list map)
  (if (null? map)
      '()
      (cons
       (cdar map)
       (index-map->index-list
	(cdr map)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; TABLES ;;;;;

;; TABLE: alist of (index . alist) pairs
;; rows/columns: lists of row/column indeces
(define (make-table rows columns)
  (letrec ([make-columns (lambda (cs)
			   (if (null? cs)
			       '()
			       (cons
				(cons (car cs) 'empty)
				(make-columns (cdr cs)))))]
	   [make-rows (lambda (rs)
			(if (null? rs)
			    '()
			    (cons
			     (cons (car rs) (make-columns columns))
			     (make-rows (cdr rs)))))]
	   [table (make-rows rows)])
    table))

;; Returns a table entry
(define (table-entry-lookup table row column)
    (cdr (assoc column (cdr (assoc row table)))))

;; Returns an alist (the "row")
(define (table-row-lookup table row)
  (cdr (assoc row table)))

(define (table-entry-install! table row column entry)
  (set-cdr! (assoc column (cdr (assoc row table))) entry))

;; Adds a new (column-index . value) pair to row
;; Mutates the 'table' argument.
(define (table-row-install! table row column val)
  (let ([list (cdr (assoc row table))]
	[pair (cons column val)])
    (set! list (cons pair list))
    (set-cdr! (assoc row table) list)))

;; Completely fill table with values.
;; Value-generator-fun must be a function taking 2 arguments (current r/c index).
(define (table-fill table rows columns value-generator-fun)
  (letrec ([fill-rows
	    (lambda (r)
	      (letrec ([fill-columns
			(lambda (c)
			  (table-entry-install!
			   table r c (value-generator-fun r c)))])
		(for-each fill-columns columns)))])
    (for-each fill-rows rows)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; ALIAS ;;;;;

;; STATES: Global index map of states
;; ( (state . int) ... )
(define (state-install! state)
  (call-with-values
      (lambda () (index-map-install! states state))
      (lambda (mstates index)
	(set! states mstates)
	index)))

(define (state-lookup state)
  (index-map-lookup states state state-eq?))

(define (state-reverse-lookup int)
  (index-map-reverse-lookup states int))

; (define state-counter 0)

; (define (state-install! state)
;   (let ((name (conc "state" (number->string state-counter)))
;         (old state-counter))
;     (hash-table-set! states name state)
;     (set! state-counter (+ 1 state-counter))
;     old))

; (define (state-lookup state)
;   (let ((ks (hash-table-keys states)))

;     (let loop ((is ks))
;       (if (null? is)
;           #f
;           (if (state-equal? state (hash-table-ref states (car is)))
;               (car is)
;               (loop (cdr is)))))))

; (define (state-reverse-lookup int)
;    (let ((name (conc "state" (number->string int))))
;      (hash-table-ref states name)))





;; RULES-TABLE:
;;         state-index ...
;; nonterm rule-number
;; ...
;;
;; ==> ( (nonterm . ((state-index . rule-number) ...))
;;        ... )

;; Reads global list 'states', 'nonterminals'.
(define (make-rules-table)
  (letrec ([sis (index-map->index-list states)]
	   [table (make-table nonterminals sis)])
    (table-fill table nonterminals sis (lambda (nt s)
					 (let* ([state (state-reverse-lookup s)]
						[rule (state-rule state nt)])
					   rule)))
    table))

(define (rules-table-lookup nonterm state)
  (table-entry-lookup rules-table nonterm state))

;; TRANSITION-TABLE:
;;          repstate-index ...
;; operator state-index
;; ...
;;
;; ==> ( (operator . ((repstate-index . state-index) ...))
;;        ... )

(define (transition-table-lookup op pstate)
  (table-entry-lookup transition-table op pstate))

(define (transition-table-install! op rstate state)
  (table-row-install! transition-table
		      op
		      rstate
		      state))

;; OP-MAP:
;;          state-index    ...
;; operator repstate-index
;; ...
;;
;; ==> ( (operator . ((state-index . repstate-index) ...))
;;        ... )

;; Takes operator and state object, returns integer index of
;; representer state to which the state maps
(define (state-to-repstate op state)
  (table-entry-lookup op-map op state))

(define (op-map-install! op state rstate)
  (table-row-install! op-map
		      op
		      state
		      rstate))

;; OP-REPS: Table of RSTATE index maps
;;
;;          repstate ...
;; operator index
;; ...
;;
;; ==> ( (operator . ((repstate . int) ...))
;;        ... )
(define (op-reps-install! op pstate)
  (let ([list (table-row-lookup op-reps op)])
    (call-with-values
	(lambda () (index-map-install! list pstate))
        (lambda (newlist newindex)
	  (set-cdr! (assoc op op-reps) newlist)
	  newindex))))

(define (op-reps-lookup op pstate)
  (let ([list (table-row-lookup op-reps op)])
    (index-map-lookup list pstate state-eq?)))

(define (op-reps-reverse-lookup op int)
  (let ([list (table-row-lookup op-reps op)])
    (index-map-reverse-lookup list int)))

;;; EOF
